int cluster[N];

#define VERBOSE

int check_correctness(void) {
    //1. Check that D[D[v]] == D[v]
    int v;
    int error1 = 0;
    for (v=0; v<N; v++) if (D[D[v]] != D[v]) {
        error1 += 1;
#ifdef VERBOSE
        printf("v=%d, D[v]=%d, D[D[v]]=%d\n", v, D[v], D[D[v]]);
#endif 
    }
    if (error1>0) {
        printf("ERROR: Check 1: %d errors found\n", error1);
        return -1;
    }
#ifdef VERBOSE
    printf("Done with Check 1\n");
#endif     
    // 2. Convert array D. choose the node with the min ID in each rooted 
    //    star and set it as the root. This eliminates non-determinism 
    for (v=0; v<N; v++) if (D[v]>v) {
        // find the min
        int min = v;
        int t;
        int dv = D[v];
        for (t=0; t<N; t++) if (D[t]==D[v] && t<v) min = t;
 
        for (t=0; t<N; t++) if (D[t]==dv) D[t]=min;
    }

#ifdef VERBOSE
    printf("Done with Check 2\n");
#endif     
    // 3. check that the span-forest size value is correct
    // NOTE: correct_spawnforest_size is initialized using magic constants
    //       based on the size of the input (the number of nodes N). If the
    //       input datasets are changed, this code needs to be updated.
    int correct_spanforest_size=-1;
    if (N==19) correct_spanforest_size = 16; 
    else if (N==50) correct_spanforest_size = 47;
    else if (N==10000) correct_spanforest_size =  9995;
    else if (N==20000) correct_spanforest_size = 11361;
    else { // unexpected input size
        printf("ERROR: Check 3: Encountered unexpected input size (N=%d)\n",N);
        return -3;
    }

    if (spanforest_size != correct_spanforest_size) {
        printf("ERROR: Check 3: spanforest_size = %d instead of %d\n", spanforest_size, 
            correct_spanforest_size);
        return -3;
    }
#ifdef VERBOSE
    printf("Done with Check 3\n");
#endif     
     
    // 4. check that all edges of the forest are in the original 
    int e;
    int srcNode, destNode;
    int error4 = 0;
    for (e=0; e<spanforest_size; e++) {
        srcNode = spanforest[e][0];
        if (srcNode<0 || srcNode >= M) { //out of bounds
            printf("ERROR: Check 4: spanforest[%d][0]=%d is out of bounds (it should be in 0<=x<%d)\n", 
                e, srcNode, M);
            return -4;
        }
        destNode = spanforest[e][1];
        if (destNode<0 || destNode >= M) { //out of bounds
            printf("ERROR: Check 4: spanforest[%d][1]=%d is out of bounds (it should be in 0<=x<%d)\n", 
                e, destNode, M);
            return -4;
        }
        int t;
        int found = 0;
        for (t=vertices[srcNode]; t < vertices[srcNode]+degrees[srcNode]; t++) {
            if (edges[t][1]==destNode) {
                found = 1; break;
            }
        }
        if (!found) {
            printf("ERROR: Check 4: edge (%d,%d) not found in original graph\n", srcNode, destNode);
            return -4;
        }
    } 
#ifdef VERBOSE
    printf("Done with Check 4\n");
#endif     
    // 5. check that there are no cycles in the spanning forest
    //    like Kruskal's MST algorithm start with each verted being a tree
    //    and keep adding edges if they connect 2 trees
    int srcRoot, destRoot;
    for (v=0; v<N; v++) cluster[v] = v;
    for (e=0; e<spanforest_size; e++) {
        srcNode  = spanforest[e][0];
        destNode = spanforest[e][1];

        srcRoot = cluster[srcNode];
        while(srcRoot != cluster[srcRoot]) srcRoot = cluster[srcRoot];

        destRoot = cluster[destNode];        
        while(destRoot != cluster[destRoot]) destRoot = cluster[destRoot];
        
        if (srcRoot != destRoot) cluster[srcRoot] = destRoot;
        else {
            printf("ERROR: Check 5: edge (%d,%d) introduces cycle in spaning forest\n",
                srcNode,destNode);
#ifdef VERBOSE
            int t;
            printf("srcRoot=%d, destRoot=%d, e=%d\n", srcRoot, destRoot, e);
            for (t=0; t<=e; t++) printf("(%d,%d), ", spanforest[t][0], spanforest[t][1]);
            printf("\n");
#endif
            return -5;
        }
    }
#ifdef VERBOSE
    printf("Done with Check 5\n");
#endif     

    printf("PASS: Your program passed the 1st correctness test!\n");
    printf("NOTE: This doesn't mean your code is correct, you also need to dump\n");
    printf("       the D array and 'diff' it with the valid result in the corresponding\n");
    printf("       data/graphx directory\n");
    return 0; // No errors
}
